package person;

import java.util.Objects;

public class Person {

    private final String firstName;
    private final String lastName;
    private final String job;
    private final int gender;
    private final int yearOfBirth;

    private Person (String firstName, String lastName, String job, int gender, int yearOfBirth) {
        // az ellenorzesek kiemelhetok egy vagy tobb privat static segedmetodusba
        if (firstName == null) {
            throw new IllegalArgumentException("first name cannot be null");
        }
        if (lastName == null) {
            throw new IllegalArgumentException("last name cannot be null");
        }
        int currentYear = 2017; // nyilvan ezt ki kellene szamolni
        if (yearOfBirth < 1900 || yearOfBirth > currentYear + 1) {
            throw new IllegalArgumentException("invalid year of birth: " + yearOfBirth);
        }
        this.firstName = firstName;
        this.lastName = lastName;
        this.job = job;
        this.gender = gender;
        this.yearOfBirth = yearOfBirth;
    }

    public static Person makePerson(String firstName, String lastName, String job, int gender, int yearOfBirth) {
        return new Person(firstName, lastName, job, gender, yearOfBirth);
    }

    @Override
    public boolean equals(Object obj) {

        // reflexivitas
        if (this == obj)
            return true;

        // ha a masik null, akkor mindig false
        if (obj == null)
            return false;

        // ha a masik osztalya kulonbozo (meg subclass meg megengedheto lenne bizonyos esetekben), akkor mindig false
        if (getClass() != obj.getClass())
            return false;
        Person other = (Person) obj;

        // ezt csak akkor tehetjuk meg, ha tudjuk, hogy a this.firstName nem lehet null (lasd konstruktor)
        if (!firstName.equals(other.firstName)) {
            return false;
        }

        // ezt csak akkor tehetjuk meg, ha tudjuk, hogy a this.lastName nem lehet null (lasd konstruktor)
        if (!lastName.equals(other.lastName)) {
            return false;
        }

        // sima != operator, ugyanis primitiv int-eket hasonlitunk, equals nem hasznalhato
        if (gender != other.gender)
            return false;

        // nincs kikenyszeritva, hogy a job lehessen null, ezert bonyolultabb a helyzet:
        if (job == null) {
            if (other.job != null)
                return false;
        } else if (!job.equals(other.job))
            return false;

        /* az elobbi rovidebben (import java.util.Objects; kell az elejere, ha ezt az alakot hasznaljuk):
         * 
         * if (!Objects.equals(job, other.job)) {
         *     return false;
         * }
         */

        if (yearOfBirth != other.yearOfBirth)
            return false;
        return true;
    }
}